/**
 * @file
 * VuoList_VuoExtrapolationMode C type definition.
 *
 * @copyright Copyright © 2012–2018 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#pragma once

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoExtrapolationMode VuoList_VuoExtrapolationMode
 * A list of @ref VuoExtrapolationMode elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoExtrapolationMode_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoExtrapolationMode elements.
 */
typedef const struct VuoList_VuoExtrapolationMode_struct { void *l; } * VuoList_VuoExtrapolationMode;
/// @}
#endif

/**
 * Creates a new list of @ref VuoExtrapolationMode elements.
 */
VuoList_VuoExtrapolationMode VuoListCreate_VuoExtrapolationMode(void);

/**
 * Creates a new list of `count` instances of `value`.
 *
 * Use this in conjunction with @ref VuoListGetData_VuoExtrapolationMode to quickly initialize a large list.
 */
VuoList_VuoExtrapolationMode VuoListCreateWithCount_VuoExtrapolationMode(const unsigned long count, const VuoExtrapolationMode value);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoExtrapolationMode VuoListCopy_VuoExtrapolationMode(const VuoList_VuoExtrapolationMode list);

/**
 * Returns the @ref VuoExtrapolationMode at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoExtrapolationMode VuoListGetValue_VuoExtrapolationMode(const VuoList_VuoExtrapolationMode list, const unsigned long index);

/**
 * Returns a pointer to a C array containing the list items.
 *
 * Use this if you need a fast way to get or change multiple list items.
 *
 * You can modify values in the list by changing them in this array.
 * Just don't attempt to access beyond the list size.
 *
 * The pointer becomes invalid if you modify the list size using other functions
 * (e.g., Insert, Prepend, Append, Cut, Remove);
 * if you use those functions, just get a new pointer by calling this function again.
 *
 * If the list has no items, returns NULL.
 *
 * The pointer remains owned by the list; don't free it.
 */
VuoExtrapolationMode *VuoListGetData_VuoExtrapolationMode(const VuoList_VuoExtrapolationMode list);

/**
 * Changes the @ref VuoExtrapolationMode at @c index.
 * Index values start at 1.
 *
 * If `expandListIfNeeded` is false: If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 *
 * If `expandListIfNeeded` is true: If the list has less than `index` elements, it is expanded to accommodate the specified `index`.
 * If `index` is 0, the list is resized to contain 1 item set to `value`.
 */
void VuoListSetValue_VuoExtrapolationMode(const VuoList_VuoExtrapolationMode list, const VuoExtrapolationMode value, const unsigned long index, bool expandListIfNeeded);

/**
 * Inserts the `VuoExtrapolationMode` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoExtrapolationMode(const VuoList_VuoExtrapolationMode list, const VuoExtrapolationMode value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoExtrapolationMode(VuoList_VuoExtrapolationMode list, const VuoExtrapolationMode value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoExtrapolationMode(VuoList_VuoExtrapolationMode list, const VuoExtrapolationMode value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoExtrapolationMode(VuoList_VuoExtrapolationMode list, const unsigned long indexA, const unsigned long indexB);

#ifdef VuoExtrapolationMode_SUPPORTS_COMPARISON
/**
 * Sorts `list`.
 */
void VuoListSort_VuoExtrapolationMode(VuoList_VuoExtrapolationMode list);

/**
 * Returns true if the two lists are equivalent.
 *
 * NULL lists are never equal to non-NULL lists (even empty lists).
 */
bool VuoList_VuoExtrapolationMode_areEqual(const VuoList_VuoExtrapolationMode a, const VuoList_VuoExtrapolationMode b);

/**
 * Returns true if `a` < `b`.
 */
bool VuoList_VuoExtrapolationMode_isLessThan(const VuoList_VuoExtrapolationMode a, const VuoList_VuoExtrapolationMode b);
#endif

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoExtrapolationMode(VuoList_VuoExtrapolationMode list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoExtrapolationMode(VuoList_VuoExtrapolationMode list);

/**
 * Returns a new list containing items from the original list in the range specified by `startIndex` and `itemCount`.
 *
 * Items in the new list are retained (not copied) from the original list.
 */
VuoList_VuoExtrapolationMode VuoListSubset_VuoExtrapolationMode(VuoList_VuoExtrapolationMode list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoExtrapolationMode(VuoList_VuoExtrapolationMode list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoExtrapolationMode(VuoList_VuoExtrapolationMode list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoExtrapolationMode(VuoList_VuoExtrapolationMode list);

/**
 * Removes the `VuoExtrapolationMode` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoExtrapolationMode(VuoList_VuoExtrapolationMode list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoExtrapolationMode(const VuoList_VuoExtrapolationMode list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoExtrapolationMode VuoList_VuoExtrapolationMode_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoExtrapolationMode_getJson(const VuoList_VuoExtrapolationMode value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoExtrapolationMode_getSummary(const VuoList_VuoExtrapolationMode value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoExtrapolationMode VuoList_VuoExtrapolationMode_makeFromString(const char *str);
char * VuoList_VuoExtrapolationMode_getString(const VuoList_VuoExtrapolationMode value);
/// @}

/**
 * @}
 */

#ifdef __cplusplus
}
#endif
